@@ -267,12 +267,12 @@
     #define arma_hot __attribute__((__hot__))
   #endif
   
-  #if __has_attribute(__minsize__)
-    #undef  arma_cold
-    #define arma_cold __attribute__((__minsize__))
-  #elif __has_attribute(__cold__)
+  #if __has_attribute(__cold__)
     #undef  arma_cold
     #define arma_cold __attribute__((__cold__))
+  #elif __has_attribute(__minsize__)
+    #undef  arma_cold
+    #define arma_cold __attribute__((__minsize__))
   #endif
   
   #if defined(__has_builtin) && __has_builtin(__builtin_assume_aligned)
@@ -401,7 +401,7 @@
   #pragma message ("WARNING: use of OpenMP disabled; compiler support for OpenMP 3.1+ not detected")
   
   #if (defined(_OPENMP) && (_OPENMP < 201107))
-    #pragma message ("NOTE: your compiler appears to have an ancient version of OpenMP")
+    #pragma message ("NOTE: your compiler has an outdated version of OpenMP")
     #pragma message ("NOTE: consider upgrading to a better compiler")
   #endif
 #endif
